#include <Object_factory.h>
#include <File.h>
#include <Flow.h>

Z_START_NAMESPACE;

class FLOW_SINH : public FLOW {
  protected :
    double _os;
    COEFF A,K,m;
  public :
    FLOW_SINH() {}
    FLOW_SINH(const FLOW_SINH&, MATERIAL_PIECE*);
    virtual ~FLOW_SINH() {}

    virtual MATERIAL_PIECE* copy_self(MATERIAL_PIECE*);
    void initialize(ASCII_FILE& file, MATERIAL_PIECE*);
    double flow_rate(double v, double crit);
    double  dflow_dv();
    double  dflow_dcrit();
};

Z_END_NAMESPACE;
  Z_USE_NAMESPACE;
  
DECLARE_OBJECT(FLOW,FLOW_SINH,sinh)

FLOW_SINH::FLOW_SINH(const FLOW_SINH& in, MATERIAL_PIECE* boss) :
 FLOW(in, boss), A(in.A, boss), K(in.K, boss), m(in.m, boss)
{}

MATERIAL_PIECE* FLOW_SINH::copy_self(MATERIAL_PIECE* mp)
{  return new FLOW_SINH(*this,mp); }

void FLOW_SINH::initialize(ASCII_FILE& file, MATERIAL_PIECE* mp)
{ FLOW::initialize(file,mp);
  for(;;) {
    STRING str=file.getSTRING();
    if( str[0]=='*' ) break;
    else if(str=="A" ) A.read(str,file,mp);
    else if(str=="K" ) K.read(str,file,mp);
    else if(str=="m" ) m.read(str,file,mp);
    else INPUT_ERROR("Unknown coefficient:"+str);
  } file.back();
}

double FLOW_SINH::flow_rate(double, double crit)
{  _os=crit;
   return A*pow(sinh(crit/K),m);
}

double FLOW_SINH::dflow_dcrit()
{ return m*A/K*pow ( sinh(_os/K), m-1 ) * cosh(_os/K) ;
}

double FLOW_SINH::dflow_dv()
{ return 0.; }
